import pytest
from unittest.mock import Mock

from crabpy.gateway.crab import (
    Gewest, Provincie,
    Gemeente, Deelgemeente, Straat,
    Huisnummer, Postkanton,
    Wegobject, Wegsegment,
    Terreinobject, Perceel,
    Gebouw, Subadres,
    Adrespositie
)


class TestCrabCachedGateway:

    @pytest.fixture(scope='function')
    def crab_gateway(self, crab_gateway, crab_client_mock):
        crab_gateway.__init__(
            crab_client_mock,
            cache_config={
                'permanent.backend': 'dogpile.cache.memory',
                'permanent.expiration_time': 86400,
                'long.backend': 'dogpile.cache.memory',
                'long.expiration_time': 3600,
                'short.backend': 'dogpile.cache.memory',
                'short.expiration_time': 600,
            }
        )
        return crab_gateway

    def test_cache_is_configured(self, crab_gateway):
        from dogpile.cache.backends.memory import MemoryBackend
        assert isinstance(
            crab_gateway.caches['permanent'].backend,
            MemoryBackend
        )
        assert crab_gateway.caches['permanent'].is_configured

    def test_list_gewesten(self, crab_gateway, crab_service):
        crab_service.ListGewesten.return_value = Mock(
            GewestItem=[
                Mock(GewestId=1, TaalCodeGewestNaam='NL', GewestNaam='Vlaams')
            ]
        )
        res = crab_gateway.list_gewesten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGewesten#1') == res

    def test_list_gewesten_different_sort(self, crab_gateway, crab_service):
        crab_service.ListGewesten.return_value = Mock(
            GewestItem=[
                Mock(GewestId=1, TaalCodeGewestNaam='NL', GewestNaam='Vlaams')
            ]
        )
        res = crab_gateway.list_gewesten(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGewesten#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListGewesten#1') == NO_VALUE

    def test_list_provincies(self, crab_gateway):
        res = crab_gateway.list_provincies(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListProvinciesByGewestId#2') == res
        assert res[0].gewest.id == 2

    def test_get_provincie_by_id(self, crab_gateway):
        res = crab_gateway.get_provincie_by_id(10000)
        assert isinstance(res, Provincie)
        assert crab_gateway.caches['permanent'].get('GetProvincieByProvincieNiscode#10000') == res

    def test_list_gemeenten_default_is_vlaanderen(self, crab_gateway,
                                                  crab_service):
        crab_service.GetGewestByGewestIdAndTaalCode.return_value = Mock(
            GewestId=2,
        )
        crab_service.ListGemeentenByGewestId.return_value = Mock(
            GemeenteItem=[
                Mock(GemeenteId=1, NISGemeenteCode=10000,
                     TaalCode='NL', TaalCodeGemeenteNaam='NL')
            ]
        )
        res = crab_gateway.list_gemeenten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGemeentenByGewestId#2#1') == res
        assert res[0].gewest.id == 2

    def test_list_gemeenten_gewest_1(self, crab_gateway, crab_service):
        crab_service.GetGewestByGewestIdAndTaalCode.return_value = Mock(
            GewestId=1,
        )
        crab_service.ListGemeentenByGewestId.return_value = Mock(
            GemeenteItem=[
                Mock(GemeenteId=1, NISGemeenteCode=10000,
                     TaalCode='NL', TaalCodeGemeenteNaam='NL')
            ]
        )
        gewest = Gewest(1)
        r = crab_gateway.list_gemeenten(gewest)
        assert isinstance(r, list)
        assert crab_gateway.caches['permanent'].get('ListGemeentenByGewestId#1#1') == r
        assert r[0].gewest.id == 1

    def test_list_gemeenten_different_sort(self, crab_gateway, crab_service):
        crab_service.GetGewestByGewestIdAndTaalCode.return_value = Mock(
            GewestId=1,
        )
        crab_service.ListGemeentenByGewestId.return_value = Mock(
            GemeenteItem=[
                Mock(GemeenteId=1, NISGemeenteCode=10000,
                     TaalCode='NL', TaalCodeGemeenteNaam='NL')
            ]
        )
        res = crab_gateway.list_gemeenten(2, 1)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGemeentenByGewestId#2#1') == res
        gewest = Gewest(2)
        r = crab_gateway.list_gemeenten(gewest, 2)
        assert isinstance(r, list)
        assert crab_gateway.caches['permanent'].get('ListGemeentenByGewestId#2#2') == r
        assert not res == r

    def test_list_gemeenten_by_provincie(self, crab_gateway, crab_service):
        crab_service.GetGewestByGewestIdAndTaalCode.return_value = Mock(
            GewestId=2
        )
        crab_service.ListGemeentenByGewestId.return_value = Mock(
            GemeenteItem=[
                Mock(GemeenteId=1, NISGemeenteCode=10000,
                     TaalCode='NL', TaalCodeGemeenteNaam='NL')
            ]
        )
        res = crab_gateway.list_gemeenten_by_provincie(10000)
        assert isinstance(res, list)
        assert crab_gateway.caches['long'].get('ListGemeentenByProvincieId#10000') == res
        provincie = Provincie(10000, 'Antwerpen', Gewest(2))
        res = crab_gateway.list_gemeenten_by_provincie(provincie)
        assert isinstance(res, list)
        assert crab_gateway.caches['long'].get('ListGemeentenByProvincieId#10000') == res

    def test_get_gemeente_by_id(self, crab_gateway, crab_service):
        crab_service.GetGemeenteByGemeenteId.return_value = Mock(
            GemeenteId=1, GewestId=1, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_gemeente_by_id(1)
        assert isinstance(res, Gemeente)
        assert crab_gateway.caches['long'].get('GetGemeenteByGemeenteId#1') == res

    def test_get_gemeente_by_niscode(self, crab_gateway, crab_service):
        crab_service.GetGemeenteByNISGemeenteCode.return_value = Mock(
            GemeenteId=1, GewestId=1, NisGemeenteCode=11001, BeginBewerking=1,
            BeginOrganisatie=1
        )
        res = crab_gateway.get_gemeente_by_niscode(11001)
        assert isinstance(res, Gemeente)
        assert crab_gateway.caches['long'].get('GetGemeenteByNISGemeenteCode#11001') == res

    def test_list_deelgemeenten_by_gemeente(self, crab_gateway, crab_service):
        crab_service.GetGemeenteByNISGemeenteCode.return_value = Mock(
            GemeenteId=1, GewestId=1, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.list_deelgemeenten_by_gemeente(45062)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListDeelgemeentenByGemeenteId#45062') == res
        gemeente = Gemeente(1, None, 45062, None)
        res = crab_gateway.list_deelgemeenten_by_gemeente(gemeente)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListDeelgemeentenByGemeenteId#45062') == res

    def test_get_deelgemeente_by_id(self, crab_gateway):
        res = crab_gateway.get_deelgemeente_by_id('45062A')
        assert isinstance(res, Deelgemeente)
        assert crab_gateway.caches['permanent'].get('GetDeelgemeenteByDeelgemeenteId#45062A') == res

    def test_list_talen(self, crab_gateway, crab_service):
        crab_service.ListTalen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_talen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListTalen#1') == res

    def test_list_talen_different_sort(self, crab_gateway, crab_service):
        crab_service.ListTalen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_talen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListTalen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListTalen#1') == NO_VALUE

    def test_list_bewerkingen(self, crab_gateway, crab_service):
        res = crab_gateway.list_bewerkingen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListBewerkingen#1') == res

    def test_list_bewerkingen_different_sort(self, crab_gateway, crab_service):
        res = crab_gateway.list_bewerkingen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListBewerkingen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListBewerkingen#1') == NO_VALUE

    def test_list_organisaties(self, crab_gateway, crab_service):
        res = crab_gateway.list_organisaties()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListOrganisaties#1') == res

    def test_list_organisaties_different_sort(self, crab_gateway, crab_service):
        res = crab_gateway.list_organisaties(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListOrganisaties#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListOrganisaties#1') == NO_VALUE

    def test_list_aardadressen(self, crab_gateway, crab_service):
        crab_service.ListAardAdressen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardadressen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardAdressen#1') == res

    def test_list_aardadressen_different_sort(self, crab_gateway, crab_service):
        crab_service.ListAardAdressen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardadressen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardAdressen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListAardAdressen#1') == NO_VALUE

    def test_list_aardgebouwen(self, crab_gateway, crab_service):
        crab_service.ListAardGebouwen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardgebouwen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardGebouwen#1') == res

    def test_list_aardgebouwen_different_sort(self, crab_gateway,
                                              crab_service):
        crab_service.ListAardGebouwen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardgebouwen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardGebouwen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListAardGebouwen#1') == NO_VALUE

    def test_list_aardwegobjecten(self, crab_gateway, crab_service):
        crab_service.ListAardWegobjecten.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardwegobjecten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardWegobjecten#1') == res

    def test_list_aardwegobjecten_different_sort(self, crab_gateway,
                                                 crab_service):
        crab_service.ListAardWegobjecten.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_aardwegobjecten(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardWegobjecten#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListAardWegobjecten#1') == NO_VALUE

    def test_list_aardterreinobjecten(self, crab_gateway, crab_service):
        crab_service.ListAardTerreinobjecten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_aardterreinobjecten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardTerreinobjecten#1') == res

    def test_list_aardterreinobjecten_different_sort(self, crab_gateway,
                                                     crab_service):
        crab_service.ListAardTerreinobjecten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_aardterreinobjecten(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListAardTerreinobjecten#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListAardTerreinobjecten#1') == NO_VALUE

    def test_list_statushuisnummers(self, crab_gateway, crab_service):
        crab_service.ListStatusHuisnummers.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statushuisnummers()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusHuisnummers#1') == res

    def test_list_statushuisnummers_different_sort(self, crab_gateway,
                                                   crab_service):
        crab_service.ListStatusHuisnummers.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statushuisnummers(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusHuisnummers#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListStatusHuisnummers#1') == NO_VALUE

    def test_list_statussubadressen(self, crab_gateway, crab_service):
        crab_service.ListStatusSubadressen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statussubadressen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusSubadressen#1') == res

    def test_list_statussubadressen_different_sort(self, crab_gateway,
                                                   crab_service):
        crab_service.ListStatusSubadressen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statussubadressen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusSubadressen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListStatusSubadressen#1') == NO_VALUE

    def test_list_statusstraatnamen(self, crab_gateway, crab_service):
        crab_service.ListStatusStraatnamen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statusstraatnamen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusStraatnamen#1') == res

    def test_list_statusstraatnamen_different_sort(self, crab_gateway,
                                                   crab_service):
        crab_service.ListStatusStraatnamen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statusstraatnamen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusStraatnamen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListStatusStraatnamen#1') == NO_VALUE

    def test_list_statuswegsegmenten(self, crab_gateway, crab_service):
        crab_service.ListStatusWegsegmenten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statuswegsegmenten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusWegsegmenten#1') == res

    def test_list_statuswegsegmenten_different_sort(self, crab_gateway,
                                                    crab_service):
        crab_service.ListStatusWegsegmenten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_statuswegsegmenten(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusWegsegmenten#2') == res

        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListStatusWegsegmenten#1') == NO_VALUE

    def test_list_geometriemethodewegsegmenten(self, crab_gateway,
                                               crab_service):
        crab_service.ListGeometriemethodeWegsegmenten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_geometriemethodewegsegmenten()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeWegsegmenten#1') == res

    def test_list_geometriemethodewegsegmenten_different_sort(
            self, crab_gateway, crab_service):
        crab_service.ListGeometriemethodeWegsegmenten.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_geometriemethodewegsegmenten(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeWegsegmenten#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeWegsegmenten#1') == NO_VALUE

    def test_list_statusgebouwen(self, crab_gateway, crab_service):
        crab_service.ListStatusGebouwen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_statusgebouwen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusGebouwen#1') == res

    def test_list_statusgebouwen_different_sort(self, crab_gateway,
                                                crab_service):
        crab_service.ListStatusGebouwen.return_value = Mock(CodeItem=[Mock()])
        res = crab_gateway.list_statusgebouwen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListStatusGebouwen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListStatusGebouwen#1') == NO_VALUE

    def test_list_geometriemethodegebouwen(self, crab_gateway, crab_service):
        crab_service.ListGeometriemethodeGebouwen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_geometriemethodegebouwen()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeGebouwen#1') == res

    def test_list_geometriemethodegebouwen_different_sort(self, crab_gateway,
                                                          crab_service):
        crab_service.ListGeometriemethodeGebouwen.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_geometriemethodegebouwen(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeGebouwen#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListGeometriemethodeGebouwen#1') == NO_VALUE

    def test_list_herkomstadrespositie(self, crab_gateway, crab_service):
        crab_service.ListHerkomstAdresposities.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_herkomstadresposities()
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListHerkomstAdresposities#1') == res

    def test_list_herkomstadrespositie_different_sort(self, crab_gateway,
                                                      crab_service):
        crab_service.ListHerkomstAdresposities.return_value = Mock(
            CodeItem=[Mock()]
        )
        res = crab_gateway.list_herkomstadresposities(2)
        assert isinstance(res, list)
        assert crab_gateway.caches['permanent'].get('ListHerkomstAdresposities#2') == res
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['permanent'].get('ListHerkomstAdresposities#1') == NO_VALUE

    def test_list_straten(self, crab_gateway, crab_service):
        crab_service.ListStraatnamenWithStatusByGemeenteId.return_value = Mock(
            StraatnaamWithStatusItem=[]
        )
        res = crab_gateway.list_straten(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['long'].get('ListStraatnamenWithStatusByGemeenteId#11') == res
        gem = Gemeente(2, None, None, None)
        r = crab_gateway.list_straten(gem)
        assert isinstance(r, list)
        assert crab_gateway.caches['long'].get('ListStraatnamenWithStatusByGemeenteId#21') == r

    def test_list_straten_different_sort(self, crab_gateway, crab_service):
        crab_service.ListStraatnamenWithStatusByGemeenteId.return_value = Mock(
            StraatnaamWithStatusItem=[]
        )
        res = crab_gateway.list_straten(1, 2)
        assert isinstance(res, list)
        assert crab_gateway.caches['long'].get('ListStraatnamenWithStatusByGemeenteId#12') == res
        gem = Gemeente(2, None, None, None)
        r = crab_gateway.list_straten(gem, 2)
        assert isinstance(r, list)
        assert crab_gateway.caches['long'].get('ListStraatnamenWithStatusByGemeenteId#22') == r
        from dogpile.cache.api import NO_VALUE
        assert crab_gateway.caches['long'].get('ListStraatnamenWithStatusByGemeenteId#11') == NO_VALUE

    def test_get_straat_by_id(self, crab_gateway, crab_service):
        crab_service.GetStraatnaamWithStatusByStraatnaamId.return_value = Mock(
            StraatnaamId=1, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_straat_by_id(1)
        assert isinstance(res, Straat)
        assert crab_gateway.caches['long'].get('GetStraatnaamWithStatusByStraatnaamId#1') == res

    def test_list_huisnummers_by_straat(self, crab_gateway, crab_service):
        crab_service.ListHuisnummersWithStatusByStraatnaamId.return_value = (
            Mock(HuisnummerWithStatusItem=[Mock(HuisnummerId=1)])
        )
        res = crab_gateway.list_huisnummers_by_straat(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByStraatnaamId#11') == res
        straat = Straat(2, None, None, None, None, None, None, None)
        r = crab_gateway.list_huisnummers_by_straat(straat)
        assert isinstance(r, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByStraatnaamId#21') == r

    def test_list_huisnummers_by_perceel(self, crab_gateway, crab_service):
        crab_service.ListHuisnummersWithStatusByIdentificatorPerceel\
            .return_value = Mock(
                HuisnummerWithStatusItem=[Mock(HuisnummerId=1)]
            )
        crab_service.GetHuisnummerWithStatusByHuisnummerId.return_value = (
            Mock(HuisnummerId=1, BeginBewerking=1, BeginOrganisatie=1)
        )
        res = crab_gateway.list_huisnummers_by_perceel('13040C1747/00G002')
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByIdentificatorPerceel#13040C1747/00G0021') == res
        perceel = Perceel('13040C1747/00H002')
        r = crab_gateway.list_huisnummers_by_perceel(perceel)
        assert isinstance(r, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByIdentificatorPerceel#13040C1747/00H0021') == r

    def test_list_huisnummers_by_straat_different_sort(self, crab_gateway,
                                                       crab_service):
        crab_service.ListHuisnummersWithStatusByStraatnaamId.return_value = (
            Mock(HuisnummerWithStatusItem=[Mock(HuisnummerId=1)])
        )
        res = crab_gateway.list_huisnummers_by_straat(1, 2)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByStraatnaamId#12') == res
        straat = Straat(2, None, None, None, None, None, None, None)
        r = crab_gateway.list_huisnummers_by_straat(straat, 2)
        assert isinstance(r, list)
        assert crab_gateway.caches['short'].get('ListHuisnummersWithStatusByStraatnaamId#22') == r

    def test_get_huisnummer_by_id(self, crab_gateway, crab_service):
        crab_service.GetHuisnummerWithStatusByHuisnummerId.return_value = Mock(
            HuisnummerId=1, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_huisnummer_by_id(1)
        assert isinstance(res, Huisnummer)
        assert crab_gateway.caches['short'].get('GetHuisnummerWithStatusByHuisnummerId#1') == res

    def test_get_huisnummer_by_nummer_and_straat(self, crab_gateway,
                                                 crab_service):
        crab_service.GetHuisnummerWithStatusByHuisnummer.return_value = Mock(
            HuisnummerId=1, BeginBewerking=1, BeginOrganisatie=1,
            Huisnummer='1'
        )
        crab_service.GetStraatnaamWithStatusByStraatnaamId.return_value = Mock(
            StraatnaamId=1, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_huisnummer_by_nummer_and_straat(1, 1)
        assert isinstance(res, Huisnummer)
        assert crab_gateway.caches['short'].get('GetHuisnummerWithStatusByHuisnummer#11') == res
        straat = Straat(1, None, None, None, None, None, None, None)
        res = crab_gateway.get_huisnummer_by_nummer_and_straat(1, straat)
        assert isinstance(res, Huisnummer)
        assert crab_gateway.caches['short'].get("GetHuisnummerWithStatusByHuisnummer#11") == res

    def test_list_postkantons_by_gemeente(self, crab_gateway, crab_service):
        crab_service.ListPostkantonsByGemeenteId.return_value = Mock(
            PostkantonItem=[Mock(PostkantonCode=1)]
        )
        res = crab_gateway.list_postkantons_by_gemeente(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['long'].get('ListPostkantonsByGemeenteId#1') == res
        gemeente = Gemeente(2, None, None, None)
        r = crab_gateway.list_postkantons_by_gemeente(gemeente)
        assert isinstance(r, list)
        assert crab_gateway.caches['long'].get('ListPostkantonsByGemeenteId#2') == r

    def test_get_postkanton_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.GetPostkantonByHuisnummerId.return_value = Mock(
            PostkantonCode=1
        )
        res = crab_gateway.get_postkanton_by_huisnummer(1)
        assert isinstance(res, Postkanton)
        assert crab_gateway.caches['short'].get('GetPostkantonByHuisnummerId#1') == res
        huisnummer = Huisnummer(1, None, None, None)
        r = crab_gateway.get_postkanton_by_huisnummer(huisnummer)
        assert isinstance(r, Postkanton)
        assert crab_gateway.caches['short'].get('GetPostkantonByHuisnummerId#1') == r

    def test_list_wegobjecten_by_straat(self, crab_gateway, crab_service):
        crab_service.ListWegobjectenByStraatnaamId.return_value = Mock(
            WegobjectItem=[Mock(IdentificatorWegobject=1,
                                AardWegobject=1)]
        )
        res = crab_gateway.list_wegobjecten_by_straat(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListWegobjectenByStraatnaamId#1') == res
        straat = Straat(2, None, None, None, None, None, None, None)
        res = crab_gateway.list_wegobjecten_by_straat(straat)
        assert crab_gateway.caches['short'].get('ListWegobjectenByStraatnaamId#2') == res

    def test_get_wegobject_by_id(self, crab_gateway, crab_service):
        crab_service.GetWegobjectByIdentificatorWegobject.return_value = Mock(
            IdentificatorWegobject='53839893', BeginBewerking=1,
            BeginOrganisatie=1
        )
        res = crab_gateway.get_wegobject_by_id("53839893")
        assert isinstance(res, Wegobject)
        assert crab_gateway.caches['short'].get('GetWegobjectByIdentificatorWegobject#53839893') == res

    def test_list_wegsegmenten_by_straat(self, crab_gateway, crab_service):
        crab_service.ListWegsegmentenByStraatnaamId.return_value = Mock(
            WegsegmentItem=[Mock()]
        )
        res = crab_gateway.list_wegsegmenten_by_straat(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListWegsegmentenByStraatnaamId#1') == res
        straat = Straat(2, None, None, None, None, None, None, None)
        r = crab_gateway.list_wegsegmenten_by_straat(straat)
        assert crab_gateway.caches['short'].get('ListWegsegmentenByStraatnaamId#2') == r

    def test_get_wegsegment_by_id(self, crab_gateway, crab_service):
        crab_service.GetWegsegmentByIdentificatorWegsegment.return_value = (
            Mock(IdentificatorWegsegment='108724', BeginBewerking=1,
                 BeginOrganisatie=1)
        )
        res = crab_gateway.get_wegsegment_by_id("108724")
        assert isinstance(res, Wegsegment)
        assert crab_gateway.caches['short'].get('GetWegsegmentByIdentificatorWegsegment#108724') == res

    def test_list_terreinobjecten_by_huisnummer(self, crab_gateway,
                                                crab_service):
        crab_service.ListTerreinobjectenByHuisnummerId.return_value = Mock(
            TerreinobjectItem=[Mock()]
        )
        res = crab_gateway.list_terreinobjecten_by_huisnummer(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListTerreinobjectenByHuisnummerId#1') == res
        huisnummer = Huisnummer(1, None, None, None)
        res = crab_gateway.list_terreinobjecten_by_huisnummer(huisnummer)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListTerreinobjectenByHuisnummerId#1') == res

    def test_get_terreinobject_by_id(self, crab_gateway, crab_service):
        crab_service.GetTerreinobjectByIdentificatorTerreinobject\
            .return_value = Mock(
                IdentificatorTerreinobject='13040_C_1747_G_002_00',
                BeginBewerking=1, BeginOrganisatie=1
            )
        res = crab_gateway.get_terreinobject_by_id("13040_C_1747_G_002_00")
        assert isinstance(res, Terreinobject)
        assert crab_gateway.caches['short'].get('GetTerreinobjectByIdentificatorTerreinobject#13040_C_1747_G_002_00') == res

    def test_list_percelen_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.ListPercelenByHuisnummerId.return_value = Mock(
            PerceelItem=[Mock()]
        )
        res = crab_gateway.list_percelen_by_huisnummer(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListPercelenByHuisnummerId#1') == res
        huisnummer = Huisnummer(1, None, None, None)
        res = crab_gateway.list_percelen_by_huisnummer(huisnummer)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListPercelenByHuisnummerId#1') == res

    def test_get_perceel_by_id(self, crab_gateway, crab_service):
        crab_service.GetPerceelByIdentificatorPerceel.return_value = Mock(
            IdentificatorPerceel='13040C1747/00G002',
            BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_perceel_by_id("13040C1747/00G002")
        assert isinstance(res, Perceel)
        assert crab_gateway.caches['short'].get('GetPerceelByIdentificatorPerceel#13040C1747/00G002') == res

    def test_list_gebouwen_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.ListGebouwenByHuisnummerId.return_value = Mock(
            GebouwItem=[Mock(IdentificatorGebouw=1)]
        )
        res = crab_gateway.list_gebouwen_by_huisnummer(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListGebouwenByHuisnummerId#1') == res
        huisnummer = Huisnummer(1, None, None, None)
        res = crab_gateway.list_gebouwen_by_huisnummer(huisnummer)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListGebouwenByHuisnummerId#1') == res

    def test_get_gebouw_by_id(self, crab_gateway, crab_service):
        crab_service.GetGebouwByIdentificatorGebouw.return_value = Mock(
            IdentificatorGebouw=1538575, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_gebouw_by_id("1538575")
        assert isinstance(res, Gebouw)
        assert crab_gateway.caches['short'].get('GetGebouwByIdentificatorGebouw#1538575') == res

    def test_list_subadressen_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.ListSubadressenWithStatusByHuisnummerId.return_value = (
            Mock(SubadresWithStatusItem=[Mock(SubadresId=1)])
        )
        res = crab_gateway.list_subadressen_by_huisnummer(129462)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListSubadressenWithStatusByHuisnummerId#129462') == res
        huisnummer = Huisnummer(129462, None, None, None)
        res = crab_gateway.list_subadressen_by_huisnummer(huisnummer)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListSubadressenWithStatusByHuisnummerId#129462') == res

    def test_get_subadres_by_id(self, crab_gateway, crab_service):
        crab_service.GetSubadresWithStatusBySubadresId.return_value = Mock(
            SubadresId=1120936, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_subadres_by_id(1120934)
        assert isinstance(res, Subadres)
        assert crab_gateway.caches['short'].get('GetSubadresWithStatusBySubadresId#1120934') == res

    def test_list_adresposities_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.ListAdrespositiesByHuisnummerId.return_value = Mock(
            AdrespositieItem=[Mock()]
        )
        res = crab_gateway.list_adresposities_by_huisnummer(1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListAdrespositiesByHuisnummerId#1') == res

    def test_list_adresposities_by_nummer_and_straat(self, crab_gateway,
                                                     crab_service):
        crab_service.ListAdrespositiesByHuisnummer.return_value = Mock(
            AdrespositieItem=[Mock()]
        )
        res = crab_gateway.list_adresposities_by_nummer_and_straat(1, 1)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListAdrespositiesByHuisnummer#11') == res

    def test_list_adresposities_by_subadres(self, crab_gateway, crab_service):
        crab_service.ListAdrespositiesBySubadresId.return_value = Mock(
            AdrespositieItem=[Mock()]
        )
        res = crab_gateway.list_adresposities_by_subadres(1120936)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListAdrespositiesBySubadresId#1120936') == res

    def test_list_adresposities_by_subadres_and_huisnummer(self, crab_gateway,
                                                           crab_service):
        crab_service.ListAdrespositiesBySubadres.return_value = Mock(
            AdrespositieItem=[Mock()]
        )
        res = crab_gateway.list_adresposities_by_subadres_and_huisnummer('A', 129462)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListAdrespositiesBySubadres#A129462') == res
        huisnummer = Huisnummer(129462, None, None, None)
        res = crab_gateway.list_adresposities_by_subadres_and_huisnummer('A', huisnummer)
        assert isinstance(res, list)
        assert crab_gateway.caches['short'].get('ListAdrespositiesBySubadres#A129462') == res

    def test_get_adrespositie_by_id(self, crab_gateway, crab_service):
        crab_service.GetAdrespositieByAdrespositieId.return_value = Mock(
            AdrespositieId=4428005, BeginBewerking=1, BeginOrganisatie=1
        )
        res = crab_gateway.get_adrespositie_by_id(4428005)
        assert isinstance(res, Adrespositie)
        assert str(crab_gateway.caches['short'].get('GetAdrespositieByAdrespositieId#4428005')) == str(res)

    def test_get_postadres_by_huisnummer(self, crab_gateway, crab_service):
        crab_service.GetPostadresByHuisnummerId.return_value = Mock(
            Postadres='Steenweg op Oosthoven 51, 2300 Turnhout'
        )
        res = crab_gateway.get_postadres_by_huisnummer(1)
        assert res == 'Steenweg op Oosthoven 51, 2300 Turnhout'
        assert str(crab_gateway.caches['short'].get('GetPostadresByHuisnummerId#1')) == str(res)

    def test_get_postadres_by_subadres(self, crab_gateway, crab_service):
        crab_service.GetPostadresBySubadresId.return_value = Mock(
            Postadres='Antoon van Brabantstraat 7 bus B, 2630 Aartselaar'
        )
        res = crab_gateway.get_postadres_by_subadres(1120936)
        assert res == 'Antoon van Brabantstraat 7 bus B, 2630 Aartselaar'
        assert str(crab_gateway.caches['short'].get('GetPostadresBySubadresId#1120936')) == str(res)



"""
This module contains utility functions for interacting with AGIV SOAP services.

.. versionadded:: 0.1.0
"""
import logging

import requests
from requests import RequestException
from suds.client import Client

log = logging.getLogger(__name__)


def crab_factory(**kwargs):
    """
    Factory that generates a CRAB client.

    A few parameters will be handled by the factory, other parameters will
    be passed on to the client.

    :param wsdl: `Optional.` Allows overriding the default CRAB wsdl url.
    :param proxy: `Optional.` A dictionary of proxy information that is passed
        to the underlying :class:`suds.client.Client`
    :rtype: :class:`suds.client.Client`
    """
    if "wsdl" in kwargs:
        wsdl = kwargs["wsdl"]
        del kwargs["wsdl"]
    else:
        wsdl = "http://crab.agiv.be/wscrab/wscrab.svc?wsdl"
    log.info("Creating CRAB client with wsdl: %s", wsdl)
    c = Client(wsdl, **kwargs)
    return c


def crab_request(client, action, *args):
    """
    Utility function that helps making requests to the CRAB service.

    :param client: A :class:`suds.client.Client` for the CRAB service.
    :param string action: Which method to call, eg. `ListGewesten`
    :returns: Result of the SOAP call.

    .. versionadded:: 0.3.0
    """
    log.debug("Calling %s on CRAB service.", action)
    return getattr(client.service, action)(*args)


class AdressenRegisterClientException(Exception):
    pass


class AdressenRegisterClient:
    def __init__(self, base_url, api_key):
        super().__init__()
        self.session = requests.Session()
        self.session.headers.update(
            {"Accept": "application/json", "x-api-key": api_key}
        )
        self.base_url = base_url[:-1] if base_url.endswith("/") else base_url

    def _get_list(self, url, response_key, params=None):
        if params is None:
            params = {}
        if "limit" not in params:
            params["limit"] = 500
        result = []
        response = {"volgende": f"{self.base_url}{url}"}
        try:
            while "volgende" in response:
                response = self.session.get(
                    response["volgende"], params=params
                )
                response.raise_for_status()
                response = response.json()
                result.extend(response[response_key])
        except RequestException as e:
            raise AdressenRegisterClientException from e
        return result

    def _get(self, url, params=None):
        try:
            response = self.session.get(f"{self.base_url}{url}", params=params)
            response.raise_for_status()
            return response.json()
        except RequestException as e:
            raise AdressenRegisterClientException from e

    def get_gemeente(self, gemeente_id):
        return self._get(f"/v1/gemeenten/{gemeente_id}")

    def get_gemeenten(self, gemeentenaam=None, status=None):
        params = {}
        if gemeentenaam is not None:
            params["gemeentenaam"] = gemeentenaam
        if status is not None:
            params["status"] = status
        return self._get_list("/v1/gemeenten", "gemeenten", params)

    def get_postinfo(self, postinfo_id):
        return self._get(f"/v1/postinfo/{postinfo_id}")

    def get_postinfos(self, gemeentenaam=None):
        params = {}
        if gemeentenaam is not None:
            params["gemeentenaam"] = gemeentenaam
        return self._get_list("/v1/postinfo", "postInfoObjecten", params=params)

    def get_straatnaam(self, straatnaam_id):
        return self._get(f"/v1/straatnamen/{straatnaam_id}")

    def get_straatnamen(
        self, straatnaam=None, gemeentenaam=None, niscode=None, status=None
    ):
        params = {}
        if straatnaam is not None:
            params["straatnaam"] = straatnaam
        if gemeentenaam is not None:
            params["gemeentenaam"] = gemeentenaam
        if niscode is not None:
            params["nisCode"] = niscode
        if status is not None:
            params["status"] = status
        return self._get_list("/v1/straatnamen", "straatnamen", params=params)

    def get_adres_match(
        self,
        gemeentenaam=None,
        niscode=None,
        postcode=None,
        kadaster_straatcode=None,
        rr_straatcode=None,
        straatnaam=None,
        huisnummer=None,
        index=None,
        busnummer=None,
    ):
        params = {}
        if gemeentenaam is not None:
            params["gemeentenaam"] = gemeentenaam
        if niscode is not None:
            params["niscode"] = niscode
        if postcode is not None:
            params["postcode"] = postcode
        if kadaster_straatcode is not None:
            params["kadStraatcode"] = kadaster_straatcode
        if rr_straatcode is not None:
            params["rrStraatcode"] = rr_straatcode
        if straatnaam is not None:
            params["straatnaam"] = straatnaam
        if huisnummer is not None:
            params["huisnummer"] = huisnummer
        if index is not None:
            params["index"] = index
        if busnummer is not None:
            params["busnummer"] = busnummer
        return self._get("/v1/adresmatch", params=params)

    def get_adres(self, adres_id):
        return self._get(f"/v1/adressen/{adres_id}")

    def get_adressen(
        self,
        gemeentenaam=None,
        postcode=None,
        straatnaam=None,
        homoniem_toevoeging=None,
        huisnummer=None,
        busnummer=None,
        niscode=None,
        status=None,
    ):
        params = {}
        if gemeentenaam is not None:
            params["gemeentenaam"] = gemeentenaam
        if postcode is not None:
            params["postcode"] = postcode
        if straatnaam is not None:
            params["straatnaam"] = straatnaam
        if homoniem_toevoeging is not None:
            params["homoniemToevoeging"] = homoniem_toevoeging
        if huisnummer is not None:
            params["huisnummer"] = huisnummer
        if busnummer is not None:
            params["busnummer"] = busnummer
        if niscode is not None:
            params["niscode"] = niscode
        if status is not None:
            params["status"] = status
        return self._get_list("/v1/adressen", "adressen", params=params)

    def get_perceel(self, perceel_id):
        return self._get(f"/v1/percelen/{perceel_id}")

    def get_percelen(self, status=None):
        params = {}
        if status is not None:
            params["status"] = status
        return self._get_list(f"/v1/percelen", "percelen", params=params)

    def get_gebouw(self, gebouw_id):
        return self._get(f"/v1/gebouwen/{gebouw_id}")

    def get_gebouwen(self, status=None):
        params = {}
        if status is not None:
            params["status"] = status
        return self._get_list(f"/v1/gebouwen", "gebouwen", params=params)



import pytest
from responses import RequestsMock

from crabpy.client import AdressenRegisterClient
from crabpy.client import (
    crab_factory,
)


@pytest.mark.skipif(
    not pytest.config.getoption("--crab-integration"),
    reason="No CRAB Integration tests required",
)
class TestCrabClient:
    def setup_method(self, method):
        self.crab = crab_factory()

    def teardown_method(self, method):
        self.crab = None

    def test_override_wsdl(self):
        wsdl = "http://crab.agiv.be/wscrab/wscrab.svc?wsdl"
        self.crab = crab_factory(wsdl=wsdl)
        assert self.crab.wsdl.url == wsdl

    def test_list_gemeenten(self):
        res = self.crab.service.ListGemeentenByGewestId(2)
        assert len(res) > 0


class TestAdressenRegisterClient:
    @pytest.fixture()
    def client(self):
        return AdressenRegisterClient("https://test-adres.be", "key")

    @pytest.fixture()
    def requests_mock(self):
        with RequestsMock() as requests_mock:
            yield requests_mock

    def test_get_gemeente(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gemeenten/123",
            json={"name": "test-gemeente"},
        )
        res = client.get_gemeente("123")
        assert res == {"name": "test-gemeente"}

    def test_get_gemeenten(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gemeenten",
            json={"gemeenten": [{"name": "test-gemeente"}]},
        )
        res = client.get_gemeenten()
        assert res == [{"name": "test-gemeente"}]

    def test_get_postinfo(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/postinfo/123",
            json={"name": "test-postinfo"},
        )
        res = client.get_postinfo("123")
        assert res == {"name": "test-postinfo"}

    def test_get_postinfos(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/postinfo",
            json={"postInfoObjecten": [{"name": "test-postinfo"}]},
        )
        res = client.get_postinfos()
        assert res == [{"name": "test-postinfo"}]

    def test_get_straatnaam(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/straatnamen/123",
            json={"name": "test-straatnaam"},
        )
        res = client.get_straatnaam("123")
        assert res == {"name": "test-straatnaam"}

    def test_get_straatnamen(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/straatnamen",
            json={"straatnamen": [{"name": "test-straatnaam"}]},
        )
        res = client.get_straatnamen()
        assert res == [{"name": "test-straatnaam"}]

    def test_get_adres(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/adressen/123",
            json={"name": "test-adres"},
        )
        res = client.get_adres("123")
        assert res == {"name": "test-adres"}

    def test_get_adressen(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/adressen",
            json={"adressen": [{"name": "test-adres"}]},
        )
        res = client.get_adressen()
        assert res == [{"name": "test-adres"}]

    def test_get_perceel(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/percelen/123",
            json={"name": "test-perceel"},
        )
        res = client.get_perceel("123")
        assert res == {"name": "test-perceel"}

    def test_get_percelen(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/percelen",
            json={"percelen": [{"name": "test-perceel"}]},
        )
        res = client.get_percelen()
        assert res == [{"name": "test-perceel"}]

    def test_get_gebouw(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gebouwen/123",
            json={"name": "test-gebouw"},
        )
        res = client.get_gebouw("123")
        assert res == {"name": "test-gebouw"}

    def test_get_gebouwen(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gebouwen",
            json={"gebouwen": [{"name": "test-gebouw"}]},
        )
        res = client.get_gebouwen()
        assert res == [{"name": "test-gebouw"}]

    def test_get_adres_match(self, client, requests_mock):
        response = {
            "adresMatches": [
                {
                    "gemeente": {
                        "objectId": "11001",
                        "detail": "https://test.be/v1/gemeenten/11001",
                        "gemeentenaam": {
                            "geografischeNaam": {
                                "spelling": "Aartselaar",
                                "taal": "nl",
                            }
                        },
                    },
                }
            ]
        }
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/adresmatch",
            json=response,
        )
        res = client.get_adres_match()
        assert res == response

    def test_list_multiple_pages(self, client, requests_mock):
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gemeenten",
            json={
                "gemeenten": [{"name": "test-gemeente1"}],
                "volgende": "https://test-adres.be/v1/gemeenten"
            },
        )
        requests_mock.add(
            method=requests_mock.GET,
            url="https://test-adres.be/v1/gemeenten",
            json={
                "gemeenten": [{"name": "test-gemeente2"}],
            },
        )
        res = client.get_gemeenten()
        assert res == [{"name": "test-gemeente1"}, {"name": "test-gemeente2"}]



from unittest.mock import Mock

import pytest

import tests
from crabpy.gateway import adressenregister
from crabpy.gateway.adressenregister import Adres
from crabpy.gateway.adressenregister import Deelgemeente
from crabpy.gateway.adressenregister import Gebouw
from crabpy.gateway.adressenregister import Gemeente
from crabpy.gateway.adressenregister import Gewest
from crabpy.gateway.adressenregister import Perceel
from crabpy.gateway.adressenregister import Provincie
from crabpy.gateway.adressenregister import Straat
from crabpy.gateway.exception import GatewayResourceNotFoundException


@pytest.fixture()
def client():
    return Mock()


@pytest.fixture()
def gateway(client):
    return adressenregister.Gateway(client)


def create_client_list_gemeenten_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/gemeente/54007",
            "naamruimte": "https://data.vlaanderen.be/id/gemeente",
            "objectId": "54007",
            "versieId": "2002-08-13T16:33:18+02:00",
        },
        "detail": "https://api.basisregisters.vlaanderen.be/v1/gemeenten/54007",
        "gemeentenaam": {
            "geografischeNaam": {"spelling": "Mouscron", "taal": "fr"}
        },
        "gemeenteStatus": "inGebruik",
    }


def create_client_get_gemeente_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/gemeente/54007",
            "naamruimte": "https://data.vlaanderen.be/id/gemeente",
            "objectId": "54007",
            "versieId": "2002-08-13T16:33:18+02:00",
        },
        "officieleTalen": ["fr"],
        "faciliteitenTalen": ["nl"],
        "gemeentenamen": [
            {"spelling": "Moeskroen", "taal": "nl"},
            {"spelling": "Mouscron", "taal": "fr"},
        ],
        "gemeenteStatus": "inGebruik",
    }


def create_client_list_straatnamen_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/straatnaam/1",
            "naamruimte": "https://data.vlaanderen.be/id/straatnaam",
            "objectId": "1",
            "versieId": "2011-04-29T13:34:14+02:00",
        },
        "detail": "https://api.basisregisters.vlaanderen.be/v1/straatnamen/1",
        "straatnaam": {
            "geografischeNaam": {"spelling": "Acacialaan", "taal": "nl"}
        },
        "straatnaamStatus": "inGebruik",
    }


def create_client_get_straatnaam_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/straatnaam/748",
            "naamruimte": "https://data.vlaanderen.be/id/straatnaam",
            "objectId": "748",
            "versieId": "2011-04-29T13:34:14+02:00",
        },
        "gemeente": {
            "objectId": "11002",
            "detail": "https://api.basisregisters.vlaanderen.be/v1/gemeenten/11002",
            "gemeentenaam": {
                "geografischeNaam": {"spelling": "Antwerpen", "taal": "nl"}
            },
        },
        "straatnamen": [{"spelling": "Edelvalklaan", "taal": "nl"}],
        "homoniemToevoegingen": [],
        "straatnaamStatus": "inGebruik",
    }


def create_client_list_adressen_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/adres/200001",
            "naamruimte": "https://data.vlaanderen.be/id/adres",
            "objectId": "200001",
            "versieId": "2011-04-29T14:51:01+02:00",
        },
        "detail": "https://api.basisregisters.vlaanderen.be/v1/adressen/200001",
        "huisnummer": "59",
        "volledigAdres": {
            "geografischeNaam": {
                "spelling": "Goorbaan 59, 2230 Herselt",
                "taal": "nl",
            }
        },
        "adresStatus": "inGebruik",
    }


def create_client_get_adres_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/adres/763445",
            "naamruimte": "https://data.vlaanderen.be/id/adres",
            "objectId": "763445",
            "versieId": "2011-04-29T14:57:43+02:00",
        },
        "gemeente": {
            "objectId": "11001",
            "detail": "https://api.basisregisters.vlaanderen.be/v1/gemeenten/11001",
            "gemeentenaam": {
                "geografischeNaam": {"spelling": "Aartselaar", "taal": "nl"}
            },
        },
        "postinfo": {
            "objectId": "2630",
            "detail": "https://api.basisregisters.vlaanderen.be/v1/postinfo/2630",
        },
        "straatnaam": {
            "objectId": "93",
            "detail": "https://api.basisregisters.vlaanderen.be/v1/straatnamen/93",
            "straatnaam": {
                "geografischeNaam": {"spelling": "Oudestraat", "taal": "nl"}
            },
        },
        "huisnummer": "27",
        "volledigAdres": {
            "geografischeNaam": {
                "spelling": "Oudestraat 27, 2630 Aartselaar",
                "taal": "nl",
            }
        },
        "adresPositie": {
            "point": {"coordinates": [150949.49, 203818.71], "type": "Point"}
        },
        "positieGeometrieMethode": "afgeleidVanObject",
        "positieSpecificatie": "gebouweenheid",
        "adresStatus": "inGebruik",
        "officieelToegekend": True,
    }


def create_client_get_perceel_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/perceel/11001B0009-00H004",
            "naamruimte": "https://data.vlaanderen.be/id/perceel",
            "objectId": "11001B0009-00H004",
            "versieId": "2004-02-13T10:12:36+01:00",
        },
        "perceelStatus": "gerealiseerd",
        "adressen": [
            {"objectId": "763445", "detail": "https://test.be/v1/adressen/763445"}
        ],
    }


def create_client_get_gebouw_item():
    return {
        "identificator": {
            "id": "https://data.vlaanderen.be/id/gebouw/5666547",
            "naamruimte": "https://data.vlaanderen.be/id/gebouw",
            "objectId": "5666547",
            "versieId": "2011-04-29T13:11:28+02:00",
        },
        "geometriePolygoon": {
            "polygon": {
                "coordinates": [[[140284.15277253836, 186724.74131567031]]],
                "type": "Polygon",
            }
        },
        "geometrieMethode": "ingemetenGRB",
        "gebouwStatus": "gerealiseerd",
        "gebouweenheden": [],
        "percelen": [
            {
                "objectId": "23052A0059-00C000",
                "detail": "https://test.be/v1/percelen/23052A0059-00C000",
            }
        ],
    }


class TestAdressenRegisterGateway:

    def test_list_gewesten(self, gateway):
        res = gateway.list_gewesten()
        assert len(res) == 3
        assert [gewest.naam for gewest in res] == [
            "Brussels Hoofdstedelijk Gewest",
            "Vlaams Gewest",
            "Waals Gewest",
        ]

    def test_get_gewest_by_id(self, gateway):
        res = gateway.get_gewest_by_id(2)
        assert isinstance(res, Gewest)
        assert res.id == 2
        assert res.naam == "Vlaams Gewest"
        assert len(res.provincies) == 5
        assert res.centroid == [138165.09, 189297.53]
        assert res.bounding_box == [22279.17, 153050.23, 258873.3, 244022.31]

    def test_list_provincies(self, gateway):
        res = gateway.list_provincies()
        assert len(res) == 5
        assert [p.niscode for p in res] == ["10000", "20001", "30000", "40000", "70000"]

    def test_get_provincie_by_id(self, gateway):
        res = gateway.get_provincie_by_id("10000")
        assert res.niscode == "10000"
        assert res.naam == "Antwerpen"

    def test_list_gemeenten_by_provincie(self, gateway, client):
        one = create_client_list_gemeenten_item()
        two = create_client_list_gemeenten_item()
        one["identificator"]["objectId"] = "10001"
        two["identificator"]["objectId"] = "20001"
        client.get_gemeenten.return_value = [one, two]
        res = gateway.list_gemeenten_by_provincie(
            Provincie("10000", "antwerpen", 2, gateway)
        )
        assert len(res) == 1
        assert res[0].niscode == "10001"

    def test_get_gewest_by_unexisting_id(self, gateway):
        with pytest.raises(GatewayResourceNotFoundException):
            gateway.get_gewest_by_id(5)

    def test_list_gemeenten_default(self, gateway, client):
        one = create_client_list_gemeenten_item()
        two = create_client_list_gemeenten_item()
        three = create_client_list_gemeenten_item()
        one["identificator"]["objectId"] = "10001"
        two["identificator"]["objectId"] = "20001"
        three["identificator"]["objectId"] = "60001"
        client.get_gemeenten.return_value = [one, two, three]
        res = gateway.list_gemeenten()
        assert len(res) == 2
        assert [gemeente.niscode for gemeente in res] == ["10001", "20001"]

    def test_get_gemeente_by_id(self, gateway, client):
        client.get_gemeente.return_value = create_client_get_gemeente_item()
        res = gateway.get_gemeente_by_id(1)
        assert res.niscode == "54007"
        assert res.naam == "Moeskroen"

    def test_get_gemeente_by_niscode(self, gateway, client):
        client.get_gemeente.return_value = create_client_get_gemeente_item()
        res = gateway.get_gemeente_by_niscode(1)
        assert res.niscode == "54007"
        assert res.naam == "Moeskroen"

    def test_list_deelgemeenten(self, gateway):
        res = gateway.list_deelgemeenten()
        assert len(res) == 1132
        assert res[0].naam == "Aartselaar"
        assert res[0].id == "11001A"
        assert res[0].gemeente_niscode == "11001"

    def test_list_deelgemeenten_by_gemeente(self, gateway):
        res = gateway.list_deelgemeenten_by_gemeente(
            Gemeente(niscode="11001", naam="Aartselaar", taal="nl", gateway=gateway)
        )
        assert len(res) == 1
        assert res[0].naam == "Aartselaar"
        assert res[0].id == "11001A"
        assert res[0].gemeente_niscode == "11001"

    def test_get_deelgemeente_by_id(self, gateway):
        res = gateway.get_deelgemeente_by_id("11001A")
        assert res.naam == "Aartselaar"
        assert res.id == "11001A"
        assert res.gemeente_niscode == "11001"

    def test_list_straten(self, gateway, client):
        client.get_straatnamen.return_value = [
            create_client_list_straatnamen_item(),
            create_client_list_straatnamen_item(),
        ]
        res = gateway.list_straten(Gemeente("Aartselaar", "11001", "nl", gateway))
        assert len(res) == 2
        assert res[0].id == "1"
        assert res[0].naam == "Acacialaan"
        assert res[0].status == "inGebruik"
        assert res[0].taal == "nl"

    def test_get_straat_by_id(self, gateway, client):
        client.get_straatnaam.return_value = create_client_get_straatnaam_item()
        res = gateway.get_straat_by_id(1)
        assert res.id == "748"
        assert res.naam == "Edelvalklaan"
        assert res.status == "inGebruik"
        assert res.taal == "nl"

    def test_list_adressen_by_straat(self, gateway, client):
        client.get_adressen.return_value = [
            create_client_list_adressen_item(),
            create_client_list_adressen_item(),
        ]
        res = gateway.list_adressen_by_straat(
            Straat("1", "inGebruik", "straatnaaam", "nl", gateway)
        )
        assert len(res) == 2
        assert res[0].id == "200001"
        assert res[0].huisnummer == "59"
        assert res[0].label == "Goorbaan 59, 2230 Herselt"
        assert res[0].status == "inGebruik"
        assert res[0].taal == "nl"

    def test_list_adressen_by_perceel(self, gateway, client):
        client.get_adres.return_value = create_client_get_adres_item()
        client.get_perceel.return_value = create_client_get_perceel_item()
        res = gateway.list_adressen_by_perceel(
            Perceel(
                id_="1",
                status="inGebruik",
                gateway=gateway,
            )
        )
        assert len(res) == 1
        assert res[0].id == "763445"

    def test_get_perceel_by_id(self, gateway, client):
        client.get_perceel.return_value = create_client_get_perceel_item()
        res = gateway.get_perceel_by_id("1")
        assert res.id == '11001B0009-00H004'

    def test_get_gebouw_by_id(self, gateway, client):
        client.get_gebouw.return_value = create_client_get_gebouw_item()
        res = gateway.get_gebouw_by_id("1")
        assert res.geojson == {
            "polygon": {
                "coordinates": [[[140284.15277253836, 186724.7413156703]]],
                "type": "Polygon",
            }
        }
        assert res.id == "5666547"
        assert len(res.percelen) == 1
        assert res.percelen[0].id == "23052A0059-00C000"
        assert res.status == "gerealiseerd"


class TestGewest:

    def test_gemeenten(self, gateway, client):
        one = create_client_list_gemeenten_item()
        two = create_client_list_gemeenten_item()
        three = create_client_list_gemeenten_item()
        one["identificator"]["objectId"] = "10001"
        two["identificator"]["objectId"] = "20001"
        three["identificator"]["objectId"] = "60001"
        client.get_gemeenten.return_value = [one, two, three]

        vlaanderen = gateway.get_gewest_by_id(2)
        res = vlaanderen.gemeenten
        assert len(res) == 2
        assert [gemeente.niscode for gemeente in res] == ["10001", "20001"]

    def test_provincies(self, gateway):
        vlaanderen = gateway.get_gewest_by_id(2)
        provincies = vlaanderen.provincies
        assert len(provincies) == 5


class TestProvincie:

    def test_gemeenten(self, gateway, client):
        one = create_client_list_gemeenten_item()
        two = create_client_list_gemeenten_item()
        one["identificator"]["objectId"] = "10001"
        two["identificator"]["objectId"] = "20001"
        client.get_gemeenten.return_value = [one, two]

        p = Provincie("10001", "Antwerpen", Gewest(2, "Vlaanderen", None, None), gateway)
        res = p.gemeenten
        assert len(res) == 1
        assert res[0].niscode == "10001"


class TestGemeente:

    def test_straten(self, gateway, client):
        client.get_straatnamen.return_value = [create_client_list_straatnamen_item()]
        g = Gemeente(niscode="1", naam="test-gemeente", gateway=gateway)
        straten = g.straten
        assert len(straten) == 1
        assert straten[0].id == "1"

    def test_provincie(self, gateway):
        g = Gemeente(niscode="1", naam="test-gemeente", gateway=gateway)
        provincie = g.provincie
        assert provincie.naam == "Antwerpen"

    def test_taal(self, gateway, client):
        client.get_gemeente.return_value = create_client_get_gemeente_item()
        g = Gemeente(niscode="1", gateway=gateway)
        assert g.taal == "nl"

    def test_naam(self, gateway, client):
        client.get_gemeente.return_value = create_client_get_gemeente_item()
        g = Gemeente(niscode="1", gateway=gateway)
        assert g.naam == "Moeskroen"

    def test_caching_and_lazy_loading(self, gateway, client):
        client.get_gemeente.return_value = create_client_get_gemeente_item()
        gemeente = Gemeente(niscode="1", gateway=gateway)
        assert gemeente.naam == "Moeskroen"
        assert gemeente.naam == "Moeskroen"
        assert client.get_gemeente.call_count == 1
        gemeente = Gemeente(niscode="1", gateway=gateway)
        assert gemeente.naam == "Moeskroen"
        assert client.get_gemeente.call_count == 2
        with tests.memory_cache():
            gemeente = Gemeente(niscode="1", gateway=gateway)
            assert gemeente.naam == "Moeskroen"
            assert gemeente.naam == "Moeskroen"
            gemeente = Gemeente(niscode="1", gateway=gateway)
            assert gemeente.naam == "Moeskroen"
            assert client.get_gemeente.call_count == 3

            gemeente = Gemeente(niscode="2", gateway=gateway)
            assert gemeente.naam == "Moeskroen"
            assert client.get_gemeente.call_count == 4


class TestDeelgemeente:

    def test_gemeente(self, gateway):
        dg = Deelgemeente(
            id_="45062A",
            naam="Sint-Maria-Horebeke",
            gemeente_niscode="45062",
            gateway=None
        )
        gemeente = dg.gemeente
        assert isinstance(gemeente, Gemeente)
        assert gemeente.niscode == "45062"


class TestStraat:

    def test_adressen(self, gateway, client):
        client.get_adressen.return_value = [
            create_client_list_adressen_item()
        ]
        straat = Straat(id_=1, naam="straatnaam", gateway=gateway)
        adressen = straat.adressen
        assert len(adressen) == 1
        assert adressen[0].id == "200001"

    def test_taal(self, gateway, client):
        client.get_straatnaam.return_value = create_client_get_straatnaam_item()
        s = Straat(id_=1, gateway=gateway)
        assert s.taal == "nl"

    def test_gemeente(self, gateway, client):
        client.get_straatnaam.return_value = create_client_get_straatnaam_item()
        s = Straat(id_=1, gateway=gateway)
        assert s.gemeente.niscode == "11002"

    def test_status(self, gateway, client):
        client.get_straatnaam.return_value = create_client_get_straatnaam_item()
        s = Straat(id_=1, gateway=gateway)
        assert s.status == "inGebruik"


class TestAdres:

    def test_adres(self, gateway, client):
        client.get_adres.return_value = create_client_get_adres_item()
        adres = Adres(id_="1", gateway=gateway)
        assert adres.label == "Oudestraat 27, 2630 Aartselaar"
        assert adres.taal == "nl"
        assert adres.huisnummer == "27"

    def test_gemeente(self, gateway, client):
        client.get_adres.return_value = create_client_get_adres_item()
        adres = Adres(id_="1", gateway=gateway)
        gemeente = adres.gemeente
        assert gemeente.niscode == "11001"


class TestPerceel:

    def test_adressen(self, gateway, client):
        client.get_perceel.return_value = create_client_get_perceel_item()
        perceel = Perceel(id_="1", gateway=gateway)
        adressen = perceel.adressen
        assert len(adressen) == 1
        assert adressen[0].id == "763445"

    def test_status(self, gateway, client):
        client.get_perceel.return_value = create_client_get_perceel_item()
        perceel = Perceel(id_="1", gateway=gateway)
        assert perceel.status == "gerealiseerd"


class TestGebouw:

    def test_gebouw(self, gateway, client):
        client.get_gebouw.return_value = create_client_get_gebouw_item()
        gebouw = Gebouw(id_=1, gateway=gateway)
        assert gebouw.status == "gerealiseerd"
        assert gebouw.geojson == {
            'polygon': {
                'coordinates': [[[140284.15277253836, 186724.7413156703]]],
                'type': 'Polygon'
            }
        }

    def test_percelen(self, gateway, client):
        client.get_gebouw.return_value = create_client_get_gebouw_item()
        gebouw = Gebouw(id_=1, gateway=gateway)
        percelen = gebouw.percelen
        assert len(percelen) == 1
        assert percelen[0].id == "23052A0059-00C000"



"""
This module contains an opionated gateway for the adressen register.

.. versionadded:: 0.14.0
"""
import json
import logging
import os

from dogpile.cache import make_region
from dogpile.util import compat

from crabpy.client import AdressenRegisterClient
from crabpy.gateway.exception import GatewayResourceNotFoundException

LOG = logging.getLogger(__name__)
AUTO = object()


LONG_CACHE = make_region()
SHORT_CACHE = make_region()


def setup_cache(cache_settings):
    if cache_settings is None:
        if not LONG_CACHE.is_configured:
            LONG_CACHE.configure("dogpile.cache.null")
        if not SHORT_CACHE.is_configured:
            SHORT_CACHE.configure("dogpile.cache.null")
    else:
        cache_settings["long.replace_existing_backend"] = True
        LONG_CACHE.configure_from_config(cache_settings, "long.")
        cache_settings["short.replace_existing_backend"] = True
        SHORT_CACHE.configure_from_config(cache_settings, "short.")


def cache_on_attribute(attribute):
    """
    Caches a method and uses an attribtue of the `self` in the caching key.

    Caching bound methods / instance methods without arguments which return
    values dependant on the attributes of the instance can not work with the
    normal `function_key_generator` from dogpile because the generated key
    would be shared across different instances.

    This method returns a `function_key_generator` which will add an
    instance attribute to the key so different instances can store their
    cached data under a different key.
    """

    def function_key_generator(namespace, fn, to_str=str):
        # Copy from dogpile.cache.util except the marked line(s)

        if namespace is None:
            namespace = f"{fn.__module__}:{fn.__name__}"
        else:
            namespace = f"{fn.__module__}:{fn.__name__}|{namespace}"

        fn_args = compat.inspect_getargspec(fn)
        has_self = fn_args[0] and fn_args[0][0] in ("self", "cls")

        def generate_key(*args, **kw):
            if kw:
                raise ValueError(
                    "dogpile.cache's default key creation "
                    "function does not accept keyword arguments."
                )
            if has_self:
                # Start of difference
                args = [args[0].__class__.__name__, getattr(args[0], attribute)] + list(
                    args[1:]
                )
                # End of difference

            return namespace + "|" + " ".join(map(to_str, args))

        return generate_key

    return function_key_generator


class LazyProperty:
    """
    A lazy property is a cached_property which can also be set a value.

    A lazy property will remember the value once it has been accessed once.
    The code inside the property will run maximum 1 time per instance.
    When the property is given a value, the code inside will never run and
    the given value will be returned when retrieving the property.
    """

    def __init__(self, method):
        super().__init__()
        self.method = method
        self.cache_name = f"_cache_{self.method.__name__}"

    def __get__(self, instance, owner):
        try:
            return getattr(instance, self.cache_name)
        except AttributeError:
            value = self.method(instance)
            setattr(instance, self.cache_name, value)
            return value

    def __set__(self, instance, value):
        setattr(instance, self.cache_name, value)


class Gateway:
    """A gateway to the adressen register."""

    def __init__(self, client: AdressenRegisterClient, cache_settings=None):
        self.client = client

        def deelgemeente_from_json_data(data):
            return Deelgemeente(
                id_=data["id"],
                naam=data["naam"],
                gemeente_niscode=data["gemeente_niscode"],
                gateway=self,
            )

        def gewest_from_json_data(data):
            return Gewest(
                id_=data["id"],
                naam=data["naam"],
                centroid=data["centroid"],
                bounding_box=data["bounding_box"],
                gateway=self,
            )

        def provincie_from_json_data(data):
            return Provincie(
                niscode=data["niscode"],
                naam=data["naam"],
                gewest=data["gewest"],
                gateway=self,
            )

        data_dir = os.path.join(os.path.dirname(__file__), "..", "data")
        with open(os.path.join(data_dir, "deelgemeenten.json"), encoding="utf-8") as f:
            self.deelgemeenten = json.load(f, object_hook=deelgemeente_from_json_data)
        with open(os.path.join(data_dir, "gewesten.json"), encoding="utf-8") as f:
            self.gewesten = json.load(f, object_hook=gewest_from_json_data)
        with open(os.path.join(data_dir, "provincies.json"), encoding="utf-8") as f:
            self.provincies = json.load(f, object_hook=provincie_from_json_data)

        setup_cache(cache_settings)

    def list_gewesten(self):
        return self.gewesten

    def get_gewest_by_id(self, id_):
        """
        Get a `gewest` by id.

        :param integer id_: The id of a `gewest`.
        :rtype: A :class:`Gewest`.
        """
        for gewest in self.gewesten:
            if gewest.id == id_:
                return gewest
        raise GatewayResourceNotFoundException()

    def list_provincies(self, gewest=2):
        """
        List all `provincies` in a `gewest`.

        :param gewest: The :class:`Gewest` for which the \
            `provincies` are wanted.
        :rtype: A :class:`list` of :class:`Provincie`.
        """
        return [
            provincie for provincie in self.provincies if provincie.gewest == gewest
        ]

    def get_provincie_by_id(self, niscode):
        """
        Retrieve a `provincie` by the niscode.

        :param str niscode: The niscode of the provincie.
        :rtype: :class:`Provincie`
        """
        niscode = str(niscode)
        for provincie in self.provincies:
            if provincie.niscode == niscode:
                return provincie
        return None

    @LONG_CACHE.cache_on_arguments()
    def list_gemeenten_by_provincie(self, provincie):
        """
        List all `gemeenten` in a `provincie`.

        :param provincie: The :class:`Provincie` for which the \
            `gemeenten` are wanted.
        :rtype: A :class:`list` of :class:`Gemeente`.
        """
        if not isinstance(provincie, Provincie):
            provincie = self.get_provincie_by_id(provincie)
        first_niscode_digit = str(provincie.niscode)[0]
        return [
            Gemeente.from_list_response(gemeente, self)
            for gemeente in self.client.get_gemeenten()
            if gemeente["identificator"]["objectId"][0] == first_niscode_digit
        ]

    @LONG_CACHE.cache_on_arguments()
    def list_gemeenten(self, gewest=2):
        """
        List all `gemeenten` in a `gewest`.

        :param gewest: The :class:`Gewest` for which the \
            `gemeenten` are wanted.
        :rtype: A :class:`list` of :class:`Gemeente`.
        """
        first_niscode_digits = [
            provincie.niscode[0]
            for provincie in self.list_provincies(gewest=gewest)
            if provincie.gewest == gewest or gewest is None
        ]
        return [
            Gemeente.from_list_response(gemeente, self)
            for gemeente in self.client.get_gemeenten()
            if gemeente["identificator"]["objectId"][0] in first_niscode_digits
        ]

    @LONG_CACHE.cache_on_arguments()
    def get_gemeente_by_id(self, gemeente_id):
        """
        Retrieve a `gemeente` by the crab id.

        :param integer gemeente_id: The niscode of the gemeente.
        :rtype: :class:`Gemeente`
        """
        return Gemeente.from_get_response(self.client.get_gemeente(gemeente_id), self)

    @LONG_CACHE.cache_on_arguments()
    def get_gemeente_by_niscode(self, niscode):
        """
        Retrieve a `gemeente` by the NIScode.

        :param integer niscode: The NIScode of the gemeente.
        :rtype: :class:`Gemeente`
        """
        return Gemeente.from_get_response(self.client.get_gemeente(niscode), self)

    @LONG_CACHE.cache_on_arguments()
    def list_deelgemeenten(self, gewest=2):
        """
        List all `deelgemeenten` in a `gewest`.

        :param gewest: The :class:`Gewest` for which the \
            `deelgemeenten` are wanted. Currently only Flanders is supported.
        :rtype: A :class:`list` of :class:`Deelgemeente`.
        """
        first_niscode_digits = [
            provincie.niscode[0]
            for provincie in self.list_provincies(gewest=gewest)
            if provincie.gewest == gewest or gewest is None
        ]
        return [
            deelgemeente
            for deelgemeente in self.deelgemeenten
            if deelgemeente.id[0] in first_niscode_digits
        ]

    @LONG_CACHE.cache_on_arguments()
    def list_deelgemeenten_by_gemeente(self, gemeente):
        """
        List all `deelgemeenten` in a `gemeente`.

        :param gemeente: The :class:`Gemeente` for which the \
            `deelgemeenten` are wanted. Currently only Flanders is supported.
        :rtype: A :class:`list` of :class:`Deelgemeente`.
        """
        if not isinstance(gemeente, Gemeente):
            gemeente = self.get_gemeente_by_niscode(gemeente)
        return [
            deelgemeente
            for deelgemeente in self.deelgemeenten
            if deelgemeente.gemeente_niscode == gemeente.niscode
        ]

    @LONG_CACHE.cache_on_arguments()
    def get_deelgemeente_by_id(self, deelgemeente_id):
        """
        Retrieve a `deelgemeente` by the id.

        :param string deelgemeente_id: The id of the deelgemeente.
        :rtype: :class:`Deelgemeente`
        """
        return next(
            (
                deelgemeente
                for deelgemeente in self.deelgemeenten
                if deelgemeente.id == deelgemeente_id
            ),
            None,
        )

    @LONG_CACHE.cache_on_arguments()
    def list_straten(self, gemeente):
        """
        List all `straten` in a `Gemeente`.

        :param gemeente: The :class:`Gemeente` for which the \
            `straten` are wanted.
        :rtype: A :class:`list` of :class:`Straat`
        """
        if not isinstance(gemeente, Gemeente):
            gemeente = self.get_gemeente_by_niscode(gemeente)
        return [
            Straat.from_list_response(straat, self)
            for straat in self.client.get_straatnamen(gemeentenaam=gemeente.naam)
        ]

    @LONG_CACHE.cache_on_arguments()
    def get_straat_by_id(self, straat_id):
        """
        Retrieve a `straat` by the Id.

        :param integer straat_id: The id of the `straat`.
        :rtype: :class:`Straat`
        """
        return Straat.from_get_response(self.client.get_straatnaam(straat_id), self)

    @SHORT_CACHE.cache_on_arguments()
    def list_adressen_by_straat(self, straat):
        """
        List all `adressen` in a `Straat`.

        :param straat: The :class:`Straat` for which the \
            `adressen` are wanted.
        :rtype: A :class:`list` of :class:`Adres`
        """
        if not isinstance(straat, Straat):
            straat = self.get_straat_by_id(straat)
        return [
            Adres.from_list_response(adres, self)
            for adres in self.client.get_adressen(straatnaam=straat.naam)
        ]

    @SHORT_CACHE.cache_on_arguments()
    def list_adressen_by_perceel(self, perceel):
        """
        List all `adressen` in a `Perceel`.

        :param perceel: The :class:`Perceel` for which the \
            `adressen` are wanted.
        :rtype: A :class:`list` of :class:`Adres`
        """
        if not isinstance(perceel, Perceel):
            perceel = self.get_perceel_by_id(perceel)
        return perceel.adressen

    @SHORT_CACHE.cache_on_arguments()
    def get_perceel_by_id(self, perceel_id):
        """
        Retrieve a `Perceel` by the Id.

        :param string perceel_id: the Id of the `Perceel`
        :rtype: :class:`Perceel`
        """
        return Perceel.from_get_response(self.client.get_perceel(perceel_id), self)

    @SHORT_CACHE.cache_on_arguments()
    def get_gebouw_by_id(self, gebouw_id):
        """
        Retrieve a `Gebouw` by the Id.

        :param str gebouw_id: the Id of the `Gebouw`
        :rtype: :class:`Gebouw`
        """
        return Gebouw.from_get_response(self.client.get_gebouw(gebouw_id), self)


class GatewayObject:

    def __init__(self, gateway):
        self.gateway: Gateway = gateway


class Gewest(GatewayObject):
    """
    A large administrative unit in Belgium.

    Belgium consists of 3 `gewesten`. Together they form the entire territory
    of the country.
    """

    def __init__(self, id_, naam, centroid, bounding_box, gateway=None):
        super().__init__(gateway=gateway)
        self.id = int(id_)
        self.naam = naam
        self.centroid = centroid
        self.bounding_box = bounding_box

    @LazyProperty
    def provincies(self):
        return self.gateway.list_provincies(gewest=self.id)

    @LazyProperty
    def gemeenten(self):
        return self.gateway.list_gemeenten(self.id)

    def __str__(self):
        if self.naam is not None:
            return f"{self.naam} ({self.id})"
        else:
            return f"Gewest {self.id}"

    def __repr__(self):
        return f"Gewest(id={self.id})"


class Provincie(GatewayObject):
    """
    The largest administrative unit within a :class:`Gewest`.

    .. versionadded:: 0.4.0
    """

    def __init__(self, niscode, naam, gewest, gateway):
        super().__init__(gateway)
        self.niscode = niscode
        self.naam = naam
        self.gewest = gewest

    @LazyProperty
    def gemeenten(self):
        return self.gateway.list_gemeenten_by_provincie(self)

    def __str__(self):
        return f"{self.naam} ({self.niscode})"

    def __repr__(self):
        return f"Provincie(niscode={self.niscode})"


class Gemeente(GatewayObject):
    """
    The smallest administrative unit in Belgium.
    """

    def __init__(self, niscode, gateway, naam=AUTO, taal=AUTO):
        super().__init__(gateway)
        self.niscode = niscode
        if naam is not AUTO:
            self.naam = naam
        if taal is not AUTO:
            self.taal = taal

    @classmethod
    def from_list_response(cls, gemeente, gateway):
        return Gemeente(
            naam=gemeente["gemeentenaam"]["geografischeNaam"]["spelling"],
            niscode=gemeente["identificator"]["objectId"],
            taal=gemeente["gemeentenaam"]["geografischeNaam"]["taal"],
            gateway=gateway,
        )

    @classmethod
    def from_get_response(cls, gemeente, gateway):
        res = Gemeente(niscode=gemeente["identificator"]["objectId"], gateway=gateway)
        res._source_json = gemeente
        return res

    @LazyProperty
    def taal(self):
        taal_nl = False
        taal_fr = False
        for naam in self._source_json["gemeentenamen"]:
            if naam["taal"] == "nl":
                taal_nl = True
            elif naam["taal"] == "fr":
                taal_fr = True
        return "nl" if taal_nl else ("fr" if taal_fr else None)

    @LazyProperty
    def naam(self):
        naam_nl = None
        naam_fr = None
        for naam in self._source_json["gemeentenamen"]:
            if naam["taal"] == "nl":
                naam_nl = naam["spelling"]
            elif naam["taal"] == "fr":
                naam_fr = naam["spelling"]
        return naam_nl or naam_fr

    @LazyProperty
    def straten(self):
        return self.gateway.list_straten(self)

    @LazyProperty
    def provincie(self):
        for p in self.gateway.provincies:
            if self.niscode[0] == p.niscode[0]:
                return p

    @LazyProperty
    @SHORT_CACHE.cache_on_arguments(
        function_key_generator=cache_on_attribute("niscode")
    )
    def _source_json(self):
        return self.gateway.client.get_gemeente(self.niscode)

    def __str__(self):
        return f"{self.naam} ({self.niscode})"

    def __repr__(self):
        return f"Gemeente(niscode={self.niscode})"


class Deelgemeente(GatewayObject):
    """
    A subdivision of a :class:`Gemeente`.

    .. versionadded:: 0.7.0
    """

    def __init__(self, id_, naam, gemeente_niscode, gateway):
        super().__init__(gateway)
        self.id = id_
        self.naam = naam
        self.gemeente_niscode = gemeente_niscode

    @LazyProperty
    def gemeente(self):
        return Gemeente(niscode=self.gemeente_niscode, gateway=self.gateway)

    def __str__(self):
        return f"{self.naam} ({self.id})"

    def __repr__(self):
        return f"Deelgemeente(id={self.id}"


class Straat(GatewayObject):
    """
    A street.

    A street object is always located in one and exactly one :class:`Gemeente`.
    """

    def __init__(self, id_, gateway, gemeente=AUTO, status=AUTO, naam=AUTO, taal=AUTO):
        super().__init__(gateway)
        self.id = id_
        if naam is not AUTO:
            self.naam = naam
        if taal is not AUTO:
            self.taal = taal
        if status is not AUTO:
            self.status = status
        if gemeente is not AUTO:
            self.gemeente = gemeente

    @classmethod
    def from_list_response(cls, straat, gateway):
        return Straat(
            id_=straat["identificator"]["objectId"],
            taal=straat["straatnaam"]["geografischeNaam"]["taal"],
            status=straat["straatnaamStatus"],
            naam=straat["straatnaam"]["geografischeNaam"]["spelling"],
            gateway=gateway,
        )

    @classmethod
    def from_get_response(cls, straat, gateway):
        res = Straat(id_=straat["identificator"]["objectId"], gateway=gateway)
        res._source_json = straat
        return res

    @LazyProperty
    def taal(self):
        taal_nl = False
        taal_fr = False
        for naam in self._source_json["straatnamen"]:
            if naam["taal"] == "nl":
                taal_nl = True
            elif naam["taal"] == "fr":
                taal_fr = True
        return "nl" if taal_nl else ("fr" if taal_fr else None)

    @LazyProperty
    def naam(self):
        naam_nl = None
        naam_fr = None
        for naam in self._source_json["straatnamen"]:
            if naam["taal"] == "nl":
                naam_nl = naam["spelling"]
            elif naam["taal"] == "fr":
                naam_fr = naam["spelling"]
        return naam_nl or naam_fr

    @LazyProperty
    def status(self):
        return self._source_json["straatnaamStatus"]

    @LazyProperty
    def gemeente(self):
        return Gemeente(
            niscode=self._source_json["gemeente"]["objectId"], gateway=self.gateway
        )

    @LazyProperty
    def adressen(self):
        return self.gateway.list_adressen_by_straat(self)

    @LazyProperty
    @SHORT_CACHE.cache_on_arguments(
        function_key_generator=cache_on_attribute("id")
    )
    def _source_json(self):
        return self.gateway.client.get_straatnaam(self.id)

    def __str__(self):
        return f"{self.naam} ({self.id})"

    def __repr__(self):
        return f"Straat(id={self.id})"


class Adres(GatewayObject):
    """
    An address.

    An address object is always located in one and exactly one :class:`Gemeente`.
    """

    def __init__(
        self,
        id_,
        gateway,
        status=AUTO,
        huisnummer=AUTO,
        label=AUTO,
        taal=AUTO,
        gemeente=AUTO,
    ):
        super().__init__(gateway=gateway)
        self.id = id_
        if label is not AUTO:
            self.label = label
        if taal is not AUTO:
            self.taal = taal
        if huisnummer is not AUTO:
            self.huisnummer = huisnummer
        if status is not AUTO:
            self.status = status
        if gemeente is not AUTO:
            self.gemeente = gemeente

    @classmethod
    def from_list_response(cls, adres, gateway):
        return Adres(
            id_=adres["identificator"]["objectId"],
            label=adres["volledigAdres"]["geografischeNaam"]["spelling"],
            taal=adres["volledigAdres"]["geografischeNaam"]["taal"],
            huisnummer=adres["huisnummer"],
            status=adres["adresStatus"],
            gateway=gateway,
        )

    @classmethod
    def from_get_response(cls, adres, gateway):
        res = Adres(id_=adres["identificator"]["objectId"], gateway=gateway)
        res._source_json = adres
        return res

    @LazyProperty
    def label(self):
        return self._source_json["volledigAdres"]["geografischeNaam"]["spelling"]

    @LazyProperty
    def taal(self):
        return self._source_json["volledigAdres"]["geografischeNaam"]["taal"]

    @LazyProperty
    def huisnummer(self):
        return self._source_json["huisnummer"]

    @LazyProperty
    def gemeente(self):
        return Gemeente(
            niscode=self._source_json["gemeente"]["objectId"], gateway=self.gateway
        )

    @LazyProperty
    @SHORT_CACHE.cache_on_arguments(
        function_key_generator=cache_on_attribute("id")
    )
    def _source_json(self):
        return self.gateway.client.get_adres(self.id)

    def __str__(self):
        return f"{self.label} ({self.id})"

    def __repr__(self):
        return f"Adres(id={self.id})"


class Perceel(GatewayObject):
    """A cadastral Parcel."""

    def __init__(self, id_, gateway, status=AUTO):
        super().__init__(gateway=gateway)
        self.id = id_
        if status is not AUTO:
            self.status = status

    @classmethod
    def from_get_response(cls, perceel, gateway):
        res = Perceel(
            id_=perceel["identificator"]["objectId"],
            gateway=gateway,
        )
        res._source_json = perceel
        return res

    @LazyProperty
    @SHORT_CACHE.cache_on_arguments(
        function_key_generator=cache_on_attribute("id")
    )
    def _source_json(self):
        return self.gateway.client.get_perceel(self.id)

    @LazyProperty
    def status(self):
        return self._source_json["perceelStatus"]

    @LazyProperty
    def adressen(self):
        return [
            Adres(id_=adres["objectId"], gateway=self.gateway)
            for adres in self._source_json["adressen"]
        ]

    def __str__(self):
        return f"Perceel {self.id}"

    def __repr__(self):
        return f"Perceel(id={self.id})"


class Gebouw(GatewayObject):
    """
    A building.
    """

    def __init__(self, id_, gateway, status=AUTO, percelen=AUTO, geojson=AUTO):
        super().__init__(gateway=gateway)
        self.id = id_
        if status is not AUTO:
            self.status = status
        if percelen is not AUTO:
            self.percelen = percelen
        if geojson is not AUTO:
            self.geojson = geojson

    @classmethod
    def from_get_response(cls, gebouw, gateway):
        res = Gebouw(id_=gebouw["identificator"]["objectId"], gateway=gateway)
        res._source_json = gebouw
        return res

    @LazyProperty
    def status(self):
        return self._source_json["gebouwStatus"]

    @LazyProperty
    def percelen(self):
        return [
            Perceel(id_=perceel["objectId"], gateway=self.gateway)
            for perceel in self._source_json["percelen"]
        ]

    @LazyProperty
    def geojson(self):
        return self._source_json["geometriePolygoon"]

    @LazyProperty
    @SHORT_CACHE.cache_on_arguments(
        function_key_generator=cache_on_attribute("id")
    )
    def _source_json(self):
        return self.gateway.client.get_gebouw(self.id)

    def __str__(self):
        return f"Gebouw {self.id}"

    def __repr__(self):
        return f"Gebouw(id={self.id})"



#!/usr/bin/env python

from setuptools import find_packages
from setuptools import setup

packages = [
    'crabpy',
]

requires = [
    'suds-py3>=1.4.4.1',
    'dogpile.cache',
    'requests'
]

setup(
    name='crabpy',
    version='0.16.1',
    description='Interact with geographical webservices by Informatie Vlaanderen.',
    long_description=open('README.rst').read() + '\n\n' +
                     open('CHANGES.rst').read(),
    author='Onroerend Erfgoed',
    author_email='ict@onroerenderfgoed.be',
    url='http://github.com/onroerenderfgoed/crabpy',
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    package_dir={'crabpy': 'crabpy'},
    include_package_data=True,
    install_requires=requires,
    entry_points="""\
      [paste.app_factory]
      main = crabpy:main""",
    license='MIT',
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



"""
This module contains custom errors that can be generated by gateways.

.. versionadded:: 0.2.0
"""


class GatewayException(Exception):
    """
    A base exception.
    """

    def __init__(self, message):
        self.message = message


class GatewayRuntimeException(GatewayException):
    """
    An exception that signifies a soap request went wrong.

    """

    soapfault = None
    """
    The soapfault that was generated by the service.
    """

    def __init__(self, message, soapfault):
        GatewayException.__init__(self, message)
        self.soapfault = soapfault


class GatewayAuthenticationException(GatewayRuntimeException):
    """
    An exception that signifies something went wrong during authentication.
    """
    pass


class GatewayResourceNotFoundException(GatewayException):
    """
    An exception that signifies that no results where found.
    """
    def __init__(self):
        GatewayException.__init__(self, 'This resource was not found.')



import pytest

from crabpy.gateway.capakey import (
    Gemeente,
    Afdeling,
    Sectie,
    Perceel
)


@pytest.fixture(scope="function")
def capakey_rest_gateway():
    from crabpy.gateway.capakey import CapakeyRestGateway
    capakey_rest_gateway = CapakeyRestGateway(
        cache_config={
            'permanent.backend': 'dogpile.cache.memory',
            'permanent.expiration_time': 86400,
            'long.backend': 'dogpile.cache.memory',
            'long.expiration_time': 3600,
            'short.backend': 'dogpile.cache.memory',
            'short.expiration_time': 600,
        }
    )
    return capakey_rest_gateway


class TestCapakeyRestCachedGateway:

    def test_cache_is_configured(self, capakey_rest_gateway):
        from dogpile.cache.backends.memory import MemoryBackend
        assert isinstance(
            capakey_rest_gateway.caches['permanent'].backend,
            MemoryBackend
        )
        assert capakey_rest_gateway.caches['permanent'].is_configured

    def test_list_gemeenten(self, capakey_rest_gateway,
                            municipalities_response):
        res = capakey_rest_gateway.list_gemeenten()
        assert isinstance(res, list)
        assert capakey_rest_gateway.caches['permanent'].get('list_gemeenten_rest#1') == res

    def test_list_gemeenten_different_sort(self, capakey_rest_gateway,
                                           municipalities_response):
        res = capakey_rest_gateway.list_gemeenten(2)
        assert isinstance(res, list)
        assert capakey_rest_gateway.caches['permanent'].get('list_gemeenten_rest#2') == res
        from dogpile.cache.api import NO_VALUE
        assert capakey_rest_gateway.caches['permanent'].get('list_gemeenten_rest#1') == NO_VALUE

    def test_get_gemeente_by_id(self, capakey_rest_gateway,
                                municipality_response):
        res = capakey_rest_gateway.get_gemeente_by_id(44021)
        assert isinstance(res, Gemeente)
        assert capakey_rest_gateway.caches['long'].get('get_gemeente_by_id_rest#44021') == res

    def test_list_afdelingen(self, capakey_rest_gateway,
                             municipalities_response,
                             municipality_department_response):
        res = capakey_rest_gateway.list_kadastrale_afdelingen()
        assert isinstance(res, list)
        assert capakey_rest_gateway.caches['permanent'].get('list_afdelingen_rest') == res

    def test_list_afdelingen_by_gemeente(self, capakey_rest_gateway,
                                         municipality_response,
                                         municipality_department_response):
        g = capakey_rest_gateway.get_gemeente_by_id(44021)
        assert capakey_rest_gateway.caches['long'].get('get_gemeente_by_id_rest#44021') == g
        res = capakey_rest_gateway.list_kadastrale_afdelingen_by_gemeente(g)
        assert isinstance(res, list)
        assert capakey_rest_gateway.caches['permanent'].get('list_kadastrale_afdelingen_by_gemeente_rest#44021#1') == res

    def test_get_kadastrale_afdeling_by_id(self, capakey_rest_gateway,
                                           department_response):
        res = capakey_rest_gateway.get_kadastrale_afdeling_by_id(44021)
        assert isinstance(res, Afdeling)
        assert res.id == 44021
        assert isinstance(res.gemeente, Gemeente)
        assert res.gemeente.id == 44021
        assert capakey_rest_gateway.caches['long'].get('get_kadastrale_afdeling_by_id_rest#44021') == res

    def test_list_secties_by_afdeling_id(self, capakey_rest_gateway,
                                         department_response,
                                         department_sections_response):
        res = capakey_rest_gateway.list_secties_by_afdeling(44021)
        assert isinstance(res, list)
        assert len(res) == 1
        assert capakey_rest_gateway.caches['long'].get('list_secties_by_afdeling_rest#44021') == res

    def test_get_sectie_by_id_and_afdeling(self, capakey_rest_gateway,
                                           department_response,
                                           department_section_response):
        a = capakey_rest_gateway.get_kadastrale_afdeling_by_id(44021)
        res = capakey_rest_gateway.get_sectie_by_id_and_afdeling('A', a)
        assert isinstance(res, Sectie)
        assert res.id == 'A'
        assert res.afdeling.id == 44021
        assert capakey_rest_gateway.caches['long'].get('get_sectie_by_id_and_afdeling_rest#A#44021') == res

    def test_list_percelen_by_sectie(self, capakey_rest_gateway,
                                     department_response,
                                     department_section_response,
                                     department_section_parcels_response):
        s = capakey_rest_gateway.get_sectie_by_id_and_afdeling('A', 44021)
        res = capakey_rest_gateway.list_percelen_by_sectie(s)
        assert isinstance(res, list)
        assert len(res) > 0
        assert capakey_rest_gateway.caches['short'].get('list_percelen_by_sectie_rest#44021#44021#A') == res

    def test_get_perceel_by_id_and_sectie(self, capakey_rest_gateway,
                                          department_response,
                                          department_section_response,
                                          department_section_parcels_response,
                                          department_section_parcel_response):
        s = capakey_rest_gateway.get_sectie_by_id_and_afdeling('A', 44021)
        percelen = capakey_rest_gateway.list_percelen_by_sectie(s)
        perc = percelen[0]
        res = capakey_rest_gateway.get_perceel_by_id_and_sectie(perc.id, s)
        assert isinstance(res, Perceel)
        assert res.sectie.id =='A'
        assert res.sectie.afdeling.id == 44021
        'get_perceel_by_id_and_sectie_rest#0001/00A000#A#44021'
        assert capakey_rest_gateway.caches['short'].get('get_perceel_by_id_and_sectie_rest#%s#A#44021' % perc.id) == res

    def test_get_perceel_by_capakey(self, capakey_rest_gateway,
                                    department_response,
                                    department_section_response,
                                    department_section_parcels_response,
                                    parcel_response):
        s = capakey_rest_gateway.get_sectie_by_id_and_afdeling('A', 44021)
        percelen = capakey_rest_gateway.list_percelen_by_sectie(s)
        perc = percelen[0]
        res = capakey_rest_gateway.get_perceel_by_capakey(perc.capakey)
        assert isinstance(res, Perceel)
        assert res.sectie.id == 'A'
        assert res.sectie.afdeling.id, 44021
        assert capakey_rest_gateway.caches['short'].get('get_perceel_by_capakey_rest#%s' % perc.capakey) == res

    def test_get_perceel_by_percid(self, capakey_rest_gateway,
                                   department_response,
                                   department_section_response,
                                   department_section_parcels_response,
                                   parcel_response):
        s = capakey_rest_gateway.get_sectie_by_id_and_afdeling('A', 44021)
        percelen = capakey_rest_gateway.list_percelen_by_sectie(s)
        perc = percelen[0]
        res = capakey_rest_gateway.get_perceel_by_percid(perc.percid)
        assert isinstance(res, Perceel)
        assert res.sectie.id == 'A'
        assert res.sectie.afdeling.id == 44021
        assert capakey_rest_gateway.caches['short'].get('get_perceel_by_capakey_rest#%s' % perc.capakey) == res

